<!--- 
	csw-image-textblock-carousel.cfc
	
	This is the core rendering for the Image Textblock Carousel widget.
	
	Version: [2018-01-17 10:00]
--->

<cfcomponent displayname="csw-imagetextblockcarousel" output="false" hint="Common Image Textblock Carousel Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			// load Resources
			Server.CommonSpot.udf.resources.loadResources("JQuery");
			Server.CommonSpot.udf.resources.loadResources("owl carousel");
			Server.CommonSpot.udf.resources.loadResources("bootstrap");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswButtonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswCardStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");
			Server.CommonSpot.udf.resources.loadResources("cswImageStyles");
			Server.CommonSpot.udf.resources.loadResources("cswImageTextblockCarousel");

			// If a custom stylesheet has been registered, go ahead and load it.
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswButtonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswButtonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCardStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCardStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswImageStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswImageStyles_custom");					
		
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswImageTextblockCarousel_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswImageTextblockCarousel_custom");
		</cfscript>
	</cffunction>


	<cffunction name="loadProperties" access="public" returntype="any" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW ImageTextblock Carousel";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;
			
			
			// 
			// Get Metadata (Core Image Textblock Carousel metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
			
			
			// Theme
			if( StructKeyExists(widgetData,"Theme") )
				props.themeClass = TRIM(widgetData.Theme);
				
			/*********************************/	
			/**** CAROUSEL PROPERTIES TAB ****/
			/*********************************/
			
			// Set default props
			props.ArrowAlignment = "";
			props.showArrows  = false;
			props.linkeditems = "";
			props.autoplayclass = "";
			
			// Slide_Arrows
			props.Slide_Arrows = getStandardField( widgetData=widgetData, field='Slide_Arrows', defValue='cswCentered' );
			if( len(props.Slide_Arrows) ) 
			{
				props.ArrowAlignment = TRIM(props.Slide_Arrows);
				props.showArrows  = true;
			}
			
			// Linked Items 
			props.LinkedItems = "";
			if( StructKeyExists(widgetData,"Linked_Items") )
				props.LinkedItems = TRIM(widgetData.Linked_Items);
			if( FindNoCase("Entire Area", props.LinkedItems) )
				props.LinkedItems = "Entire Area";
				
			// Options			
			props.showdots = false;
			props.infiniteloop = false;
			props.autoheight = false;
			props.autoplayHoverPause = false;
			
			if( StructKeyExists(widgetData,"options") ) 
			{
				if( FindNoCase("showdots",widgetData.options) )
					props.showdots = true; 
					
				if( FindNoCase("infiniteloop",widgetData.options) )
					props.infiniteloop = true;
					
				if( FindNoCase("autoheight",widgetData.options) )
					props.autoheight = true;
				
				if( FindNoCase("autoplayHoverPause",widgetData.options) )
					props.autoplayHoverPause = true;		
			}

			// Autoplay
			props.autoplayToggle = getStandardField( widgetData=widgetData, field='Autoplay', defValue='0' );

			if( props.autoplayToggle eq '' )
				props.autoplayToogle = '0';

			// Slide Wait Time
			props.slideWaitTime = getStandardField( widgetData=widgetData, field='slideWaitTime', defValue='0' );
			
			props.autoPlayClass = '';
			if( props.autoplayToggle eq '1' )
				props.autoPlayClass = 'cswAutoplay';
			
			// SpaceBetween
			props.SpaceBetween = getStandardField( widgetData=widgetData, field='SpaceBetween', defValue='10' );
			
			
			
			/**********************************/	
			/**** GENERAL PROPERTIES TAB *****/
			/**********************************/

			// Element
			props.element = getFieldSettings(widgetData=widgetData, field='Overall', defShow='no', defBgColor='' );	

			// Responsive Columns (STRUCT)
			props.Columns = getDeviceSizeOptions(widgetData=widgetData, field='Columns', defXSS='cswColXXS-1', defXS='cswColXS-1', defSM='cswColSM-2', defMD='cswColMD-3', defLG='cswColLG-4');
			props.colClass = getDeviceSizeClassList(props.Columns);
			props.colClass = replacenocase(props.colClass," ",",","ALL");
			
			// Background Image
			props.backgroundImage = getStandardMetadataImageField( widgetData=widgetData, field='Background_Image', defValue='' );		
			
			// Background Image Focal Point
			props.backgroundImageFocalPoint = getStandardField( widgetData=widgetData, field='Background_Image_Focal_Point', defValue='' );		
			


			/******************************/	
			/**** CARD PROPERTIES TAB *****/
			/******************************/			

			// Card (STRUCT)
			props.card = getFieldSettings(widgetData=widgetData, field='Card', defShow='1');

			// Text Padding
			props.textPadding = getStandardField( widgetData=widgetData, field='Text' );
			
			// ImageHeightPercentage
			props.ImageHeightPercentage = getStandardField( widgetData=widgetData, field='ImageHeightPercentage', defValue='100' );
			
			// Image Effect (Movement)
			props.imageMovement = getStandardField( widgetData=widgetData, field='Image_Effect', defValue='' );
			
			
			
			/*********************************************/			
 			/**** TITLE & DESCRIPTION PROPERTIES TAB *****/
			/*********************************************/
			
			// Title (STRUCT)
			props.title = getFieldSettings(widgetData=widgetData, field='Title', defShow='1', defTag='h2', defColor='', defAlign='');
			
			// Description (STRUCT)
			props.desc = getFieldSettings(widgetData=widgetData, field='Description', defShow='1', defTag='', defColor='', defSize='');
			
			
			/********************************/		
			/**** BUTTON PROPERTIES TAB *****/
			/********************************/	
			
			// Link Text					
			props.linkText = getStandardField(widgetData=widgetData, field='linkText', defValue='');	

			// Button Type
			props.buttonType = getStandardField( widgetData=widgetData, field='Button_Type', defValue='cswButton' );

			// Button (STRUCT)
			props.button = getFieldSettings(widgetData=widgetData, field='Button', defShow='1', defColor='white', defBgColor='black', defSize='cswBtnSize-normal', defStyle='cswBtn-filled');

			// Button_Corners
			props.buttonCorners = getStandardField( widgetData=widgetData, field='Button_Corners', defValue='cswBtn-rounded' );
			
			// Link (STRUCT)
			props.link = getFieldSettings(widgetData=widgetData, field='Link', defShow='0', defColor='inherit', defSize='cswBtnSize-normal', defStyle='');
			
			// Icon 
			props.buttonIcon = getStandardField( widgetData=widgetData, field='Icon' );
						
			return props;
		</cfscript>

	</cffunction>
	
	
	
	<!--- // Main function called from render handler --->
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="elementUniqueID" type="string" required="Yes">
		
		<cfscript>
			var elementID = arguments.elementUniqueID;
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;
			var items = ArrayNew(1);	
			var itm	= "";
			var totalItems = 0;
			var styleAttr = '';
			var outerClasses = '';

			// Get Records from the Element as an Array of Structures
			if( StructKeyExists(elementData, 'PropertyValues') AND ArrayLen(elementData.PropertyValues) )
				items = elementData.PropertyValues;
			
			totalItems = ArrayLen(items);
			
			styleAttr = '';
			if( structKeyExists(props,'element') )
			{
				if( len(props.element.padding) )
					styleAttr = styleAttr & 'padding: #props.element.padding#;';
				if( len(props.element.bgColor) )
					styleAttr = styleAttr & 'background-color: #props.element.bgColor#;';
			}
			if( StructKeyExists(props,'backgroundImage') AND len(props.backgroundImage) )
				styleAttr = styleAttr & "background-image: url('#props.backgroundImage#'); background-repeat: no-repeat; background-size: cover;";

			if( StructKeyExists(props,'backgroundImageFocalPoint') )
				outerClasses = trim( outerClasses & ' #props.backgroundImageFocalPoint#');	
			
			
			if( len(styleAttr) )
				styleAttr = 'style="#styleAttr#"';
		</cfscript>
		 
		<cfif arrayLen(Items)>
			<cfoutput>
			<div class="cswImageTextBlockOuter #outerClasses#" #styleAttr#>
				<div class="csw-image-textblock-carousel #props.ArrowAlignment#">
					<div class="owl-carousel Carousel#elementID# cswItems #props.autoPlayClass#">
						<!--- // Run function to render all of the slides in the carousel --->
						#RenderItems(items, props, 'all')#
					</div> 
					
					<!--- OWL Next/Previous Controls and DOTS --->
					<div class="owl-theme Carousel#elementID#">
						<div class="owl-controls">
							<div class="owl-nav customNav #props.ArrowAlignment#"></div>
							<div class="owl-dots customDots"></div>
						</div>
					</div>
				</div>
			</div>
			</cfoutput>
		</cfif> 
		
		<!--- // Render the required javascript --->
 		<cfset args = "#ArrayLen(items)#,'#elementID#',#props.infiniteloop#,#props.autoplayToggle#,#props.slideWaitTime#,#props.autoplayHoverPause#,#props.autoHeight#,#props.showArrows#,#props.showDots#,#props.colClass#,#props.SpaceBetween#">
		 
		<cfsavecontent variable="cswCarouselJS">
		 	<cfoutput>
			jQuery(document).ready(function() {
				initImageTextblockCarousel(#args#);
				jQuery(window).on('resize', function(){ resizeCarouselTextblocks('#elementID#',#props.autoheight#,1) } );
				setTimeout( function(){ resizeCarouselTextblocks('#elementID#',#props.autoheight#,0)}, 200 );
			});
			</cfoutput>
		</cfsavecontent>  
		
		<cfscript>
			// Load custom javascript at the bottom of the page.
			Server.CommonSpot.udf.resources.addFooterJS(cswCarouselJS, 'TERTIARY');
		</cfscript> 

	</cffunction>
	
	
	
	<!----------------------------------------
		RenderItems: 
			loops over the data from the element and calls function "getItemData" 
			which puts the data into a consistent format used by the widgets.  It then calls
			the "renderSlide" function to render an individual OWL slide
	----------------------------------------->  
	<cffunction name="RenderItems" access="private" output="Yes" returntype="void">
		<cfargument name="itemsArray" type="array" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="mode" type="string" required="Yes">
		<cfscript>
			var i = 1;
			var start = 1;
			var totalItems = 1;
			var items = arguments.itemsArray;
			var props = arguments.propsStruct;
			
			// Determine Mode
			if( arguments.mode eq 'first' )
			{
				start = 1;
				totalItems = 1;
			}
			else if( arguments.mode eq 'rest' )
			{
				start = 2;
				totalItems = ArrayLen(items);
			}
			else
			{
				start = 1;
				totalItems = ArrayLen(items);
			}
			
			// Loop appropriately
			for( i=start; i lte totalItems; i=i+1 )
			{
				// Get Data
				data = getItemData(items[i].values);

				// RENDER the slide
				renderSlide( data, props, i );
			}
		</cfscript>

	</cffunction>
	
	
	<!----------------------------------------
	 	 getItemData: 
		 	puts the element data into a consistent format used by the widgets  
	----------------------------------------->
	<cffunction name="getItemData" access="public" output="Yes" returntype="struct">
		<cfargument name="valuesStruct" type="struct" required="Yes">
		
		<cfscript>
			var data = StructNew();
			var values = arguments.valuesStruct;

			// Image Properties 
			// -----------------
			
			// Image - could be straight URL or Structure
			data.image = getStandardCustomElementImageField(widgetData=values, field='image', defValue='');

			// Image Focal Point (optional)
			data.imageFocalPoint = getStandardField(widgetData=values, field='Image_Focal_Point', defValue='cswImgPos-centermiddle');


			// Title & Description
			// -----------------
			
			// Title
			data.title = getStandardField(widgetData=values, field='title', defValue='');

			// Description
			data.desc = getStandardField(widgetData=values, field='description', defValue='');

			// Subtitle
//			data.subtitle = getStandardField(widgetData=values, field='subtitle', defValue='');


			// Link/Button Properties
			// -----------------
			
			// LINK URL 	
			data.linkURL = getStandardCustomElementLinkField(widgetData=values, field='LinkURL', defValue='');

			// Link Url Target
			data.LinkTarget = getStandardCustomElementLinkFieldTarget(widgetData=values, field='LinkURL', defValue='');

			// Link Url OnClick
			data.onClick = getStandardCustomElementLinkFieldOnClick(widgetData=values, field='LinkURL', defValue='');

			// Link Text					
			data.linkText = getStandardField(widgetData=values, field='linkText', defValue='' );

			// Button Type				
			data.buttonType = getStandardField( widgetData=values, field='Button_Type', defValue='' );
			
			// Button (STRUCT - returns Empty Struct if not mapped ) 
			data.button = getDataFieldSettings(widgetData=values, field='Button', defShow='', defAlign='', defSize='', defColor='', defBgColor='', defEffects='', defStyle='', defTag='', defFontSettings='');
			
			// LINK (STRUCT - returns Empty Struct if not mapped ) 
			data.link = getDataFieldSettings(widgetData=values, field='Link', defShow='', defAlign='', defSize='', defColor='', defBgColor='', defEffects='', defStyle='', defTag='', defFontSettings='');
		
			// Button Corners
			data.buttonCorners = getStandardField( widgetData=values, field='Button_Corners', defValue='' );	

			// Icon
			data.buttonIcon = getStandardField( widgetData=values, field='Icon', defValue='' );
			
			return data;
		</cfscript>
		
	</cffunction>
	
	
	
	<!----------------------------------------
		Render renderSlide 	
			renders an OWL slide
	----------------------------------------->
	<cffunction name="renderSlide" access="private" output="Yes" returntype="void">
		<cfargument name="dataStruct" required="Yes" type="struct">
		<cfargument name="propsStruct" required="Yes" type="struct">
		<cfargument name="itemcnt" required="Yes" type="numeric">
	
		<cfscript>
			var i = 0;
			var props = arguments.propsStruct;
			var data = arguments.dataStruct;
			var elementID = request.element.id;
			var itm = arguments.itemcnt;
			var slideLinkedItems = '';
			var bgPos = 'cswBgMiddleCenter';
			var cardStyles = '';
			var cardClasses = '';
			var textStyleAttr = '';
			var wrapperStyles = '';
			var wrapperClasses = '';
			
			if( len(data.linkURL))		
				slideLinkedItems = props.LinkedItems;

			if( len(data.ImageFocalPoint))
				bgPos = data.ImageFocalPoint;	
				
			// Card styles
			// ---------------
			if( len(props.card.style) AND props.card.style neq 'cswShadow-none' )
				cardClasses = cardClasses & ' #props.card.style#';

			if( len(props.card.align) )
				cardClasses = trim(cardClasses & ' #props.card.align#');

			if( len(props.card.borderColor) )
			{
				if( len(props.card.size) eq 0 )
					props.card.size = '1px';
				if( props.card.size != 'none' )
					cardStyles = cardStyles & 'border:#props.card.size# solid #props.card.borderColor#;';
			}

			if( FindNoCase('edgeup', props.card.style) AND ( len(props.card.bgColor) EQ 0 OR props.card.bgColor EQ 'inherit') )
			{
				// force background to be white (if not set and turn up shadow is set)
				wrapperStyles = wrapperStyles & 'background-color:white;';
			}			
			else if( len(props.card.bgColor) )
				wrapperStyles = wrapperStyles & 'background-color:#props.card.bgColor#;';

			if( len(props.card.padding) )
				wrapperStyles = wrapperStyles & 'padding:#props.card.padding#;';
				
			if( len(cardStyles) )
				cardStyles = 'style="#cardStyles#"';
			if( len(wrapperStyles) )
				wrapperStyles = 'style="#wrapperStyles#"';				
				
			// Text Padding
			if( len(props.textPadding) )
				textStyleAttr = 'style="padding:#props.textPadding#;"';
		</cfscript>
		
		<cfoutput>
		<div class="item">  
			<div class="cswCard #cardClasses#" #cardStyles#>
				<cfif FindNoCase("Entire Area", slideLinkedItems)>
					<a href="#data.LinkURL#" class="cswEntireAreaLink">
				</cfif>
				<div class="cswOverlayWrapper #wrapperClasses#" #wrapperStyles#>
				
					<!--- // IMAGE // --->
					<cfif FindNoCase("Image", slideLinkedItems)>
						<a href="#data.LinkURL#" class="cswImageLink">
							<div class="cswImageWrapper #props.imageMovement#">
								<div class="cswImage #bgPos#" style="padding-top:#props.ImageHeightPercentage#%; background-image:url(#data.image#);"></div>
							</div>
						</a>
					<cfelse>
						<div class="cswImageWrapper #props.imageMovement#">
							<div class="cswImage #bgPos#" style="padding-top:#props.ImageHeightPercentage#%; background-image:url(#data.image#);"></div>
						</div>
					</cfif>
					
					<div class="cswContentWrapper" #textStyleAttr#>
						<!--- // TITLE   --->
						<cfif FindNoCase("Title", slideLinkedItems)>
							<a href="#data.LinkURL#" class="cswTitleLink">
								#renderTitle( data, props )#
							</a>
						<cfelse>
							#renderTitle( data, props )#
						</cfif>  
						
						<!--- // DESCRIPTION   --->
						<cfif FindNoCase("Description", slideLinkedItems)>
							<a href="#LinkURL#" class="cswDescLink">
								#renderDescription( data, props )#
							</a>
						<cfelse>
							#renderDescription(data, props)#
						</cfif>
	
						<!--- // BUTTON --->
						#RenderButton(data,props)#
					</div>
				</div>
				<cfif FindNoCase("Entire Area",slideLinkedItems)>
					</a>
				</cfif>
			</div>
		</div>
		</cfoutput>
	</cffunction>
	
</cfcomponent>

